// This file has been prepared for Doxygen automatic documentation generation.
/*! \file ********************************************************************
*
* Atmel Corporation
*
* \li File:               main.c
* \li Compiler:           IAR EWAAVR 3.20a
* \li Support mail:       avr@atmel.com
*
* \li Supported devices:  All AVRs.
*
* \li Application Note:   AVR318 - Dallas 1-Wire(R) master.
*                         
*
* \li Description:        Example on how to use the 1-Wire(R) interface
*                         master.
*
*                         $Revision: 1.7 $
*                         $Date: Thursday, August 19, 2004 14:27:18 UTC $
****************************************************************************/

//#include <ioavr.h>
//#include <inavr.h>

#include "main.h"

#include "OWIPolled.h"
#include "OWIHighLevelFunctions.h"
#include "OWIBitFunctions.h"
#include "OWIcrc.h"

#include <string.h> // Used for memcpy.


// Defines used only in code example.
#define DS1820_FAMILY_ID				0x10 
#define DS1820_START_CONVERSION			0x44
#define DS1820_READ_SCRATCHPAD			0xbe
#define DS1820_READ_POWER_SUPPLY		0xb4
#define DS1820_ERROR					-1000   // Return code. Outside temperature range.

#define OWI_ROM_SKIP					0xcc	//!< SKIP ROM command code.

#define DS2890_FAMILY_ID				0x2c
#define DS2890_WRITE_CONTROL_REGISTER	0X55
#define DS2890_RELEASE_CODE				0x96
#define DS2890_WRITE_POSITION			0x0f

#define SEARCH_SUCCESSFUL				0x00
#define SEARCH_CRC_ERROR				0x01

#define FALSE		0
#define TRUE		1



/*! \brief  Data type used to hold information about slave devices.
 *  
 *  The OWI_device data type holds information about what bus each device
 *  is connected to, and its 64 bit identifier.
 */


// Prototypes of functions used in exemples.

unsigned char OWI_ComputeCRC8(unsigned char inData, unsigned char seed);

void	SearchBuses				(unsigned char buses);
void	DS1820_ReadTemperature	(unsigned char nr);


/*! \brief  Example application for the polled drivers.
 *
 *  Example application for the software only and polled UART driver.
 *  This example application will find all devices (upper bounded by MAX_DEVICES) 
 *  on the buses defined by BUSES. It then tries to find either a DS1820 or DS2890 
 *  device on a bus, and communicate with them to read temperature (DS1820) or 
 *  set wiper position (DS2890).
 *  This example is not written in a very optimal way. It is merely intended to show
 *  how the polled 1-Wire(R) driver can be used.
 */
void OWImain(void)
{
	unsigned char i;
	
	BUSES = OWI_PIN_6;		//!< Buses to search. (evtl. mehrere durch | verknpfen)
	DALLAS[0][0] = BUSES;	// Wert fr den HTTP-Server
	VA4 = DALLAS[0][0] & OWI_PIN_4;
	VA5 = DALLAS[0][0] & OWI_PIN_5;
	VA6 = DALLAS[0][0] & OWI_PIN_6;
	
	OWI_Init(BUSES);
	
	for(i = 1; i < MAX_DEVICES; i++) {					// Datenfeld im SRAM initialisieren
		DALLAS[i][1] = 0xff;								// Baustein ist kein Thermometer
		if (eeprom_read_byte ((char*)(i<<4)+7)==0xff)	// Wenn der Platz leer ist,
			DALLAS[i][0] = 0x7f;							//  "Eintrag ist leer" setzen
			else DALLAS[i][0] = 126;
	}

	for (;;)	// Do something useful with the slave devices in an eternal loop.
	{	
		BUSES = DALLAS[0][0];										// Busleitungen aktualisieren
	
//		for(i=1; i<MAX_DEVICES; i++) if (DALLAS[i][0]==0x7f) printf("-"); else printf("+");
//		printf("\n");	// Testausgabe
//		for(i=1; i<MAX_DEVICES; i++) if (DALLAS[i][0]==0x7f) printf("ok"); else printf("%02X",DALLAS[i][0]);
//		printf("\n");	// Testausgabe

		SearchBuses(BUSES); 	// Do the bus search.   
		OWI_DetectPresence(BUSES);									// Reset
		OWI_SendByte(OWI_ROM_SKIP, BUSES);							// Send skip ROM command. "An Alle"
		OWI_SendByte(DS1820_START_CONVERSION, BUSES);				// Send start conversion command.
		OWI_STRONG_PULLUP(BUSES);									// switch strong pullup on.
		for (i=0; i<80; i++) {__delay_cycles(OWI_DELAY_10ms);}	// wait 800 ms
		for (i=1; i<MAX_DEVICES; i++) DS1820_ReadTemperature(i);	// Thermometer auslesen
	}
}



/*! \brief  Perform a 1-Wire search
 *
 *  This function shows how the OWI_SearchRom function can be used to 
 *  discover all slaves on the bus. It will also CRC check the 64 bit
 *  identifiers.
 *
 *  \param  devices Pointer to an array of type OWI_device. The discovered 
 *                  devices will be placed from the beginning of this array.
 *
 *  \param  len     The length of the device array. (Max. number of elements).
 *
 *  \param  buses   Bitmask of the buses to perform search on.
 *
 *  \retval SEARCH_SUCCESSFUL   Search completed successfully.
 *  \retval SEARCH_CRC_ERROR    A CRC error occured. Probably because of noise
 *                              during transmission.
 */
void SearchBuses(unsigned char buses)
{
    int i;
    unsigned char * currentID;
    unsigned char j, presence, currentBus, lastDeviation, id[8];
	
	presence = OWI_DetectPresence(BUSES);					// Find slave devices on all buses.
	currentID = id;											// currentID zeigt auf das Datenfeld
	for (currentBus=0x01; currentBus; currentBus<<=1){	// Go through all buses with slave devices.
		lastDeviation = 0;									// Startbedingungen fr den Bus
		if ((currentBus&BUSES) & presence){ 			// If (Bus included in BUSES) and Devices available on any bus.
            do {  // Do slave search on each bus, and place identifiers and corresponding bus "addresses" in the EEPROM.
				OWI_DetectPresence(currentBus);				// Reset auf currentBus auslsen
				lastDeviation = OWI_SearchRom(currentID, lastDeviation, currentBus);	// nchsten Baustein suchen


//
// Jetzt machen wir den CRC-Check und falls wir am Ende der Daten im EEPROM ankommen, wird (newID) ins EEPROM eintragen.
// Den EEPROM-Index von newID merken wir uns im RAM.
if (!OWI_ComputeCRCn(id,8)) {			// Wenn CRC ok, suchen wir currentID im EEPROM
	i = 1;								// Bausteinindex (geht bei 1 los, 0 bleibt unbenutzt!)
	j = 0;								// Index im ID-Feld
	do {								// Fr alle Pltze im EEPROM wiederholen
		i <<= 4;						// Bausteinindex in EEPROM-Adresse umrechnen
		do {							// Fr alle Bytes der ID wiederholen
			if (id[j]==eeprom_read_byte((char *)i+j))	// wenn die Bytes gleich sind,
				j++;										// nchstes Byte vergleichen
			else j=99;					// anderfalls, Vergleich dieser Seriennummer abbrechen
		} while (j<7);				// Es sind 7 Byte zu vergleichen (0..6)
			
		if (j==7) {					// Die Seriennummer wurde gefunden, fertig
			if (eeprom_read_byte ((char *)i+7)!=currentBus)	// Wenn sich der Bus gendert hat
				eeprom_write_byte ((uint8_t *)i+7,currentBus);	// Bitmaske erneuern
//			i >>= 4;					// Adresse zurck auf den Bausteinindex setzen
//			if (DALLAS[i][0]==0x7f) DALLAS[i][0]=0;	// Wenn bisher nicht aktiv, Baustein vorhanden
		}
		else { 						// wenn j<>7, wurde die Seriennummer nicht gefunden
			if (eeprom_read_byte ((char *)i+7)==0xff) {		// Wenn dieser Platz leer ist,
				eeprom_write_block(currentID,(char *)i,7);	//  die ID ins EEPROM schreiben
				eeprom_write_byte ((uint8_t *)i+7,currentBus);	//  und die Bitmaske der Portleitung anhngen
				j = 7;					// Die Seriennummer steht nun im EEPROM
				//i >>= 4;				// Adresse zurck auf den Bausteinindex setzen (geschenkt)
			}
			else {						// Platz in Gebrauch, weiter nach freiem Platz suchen
				i >>= 4;				// dazu Adresse zurck auf den Bausteinindex setzen
				i += 1;					// Bausteinindex erhhen
				if (i < MAX_DEVICES)	// wenn noch Platz im EEPROM ist, weiteren Suchdurchlauf vorbereiten
					j = 0;				// Index auf den Anfang des ID-Feldes setzen
			}
		}
	} while (j==0);		// weitersuchen bis j<>0, d.h. Seriennummer gefunden oder Speicher erschpft
} // Ende der Suche und des Speicherns im EEPROM
//



			}	while(lastDeviation != OWI_ROM_SEARCH_FINISHED);            
		}
	}
}


/*! \brief  Read the temperature from a DS1820 temperature sensor.
 *
 *  This function will start a conversion and read back the temperature
 *  from a DS1820 temperature sensor.
 *
 *  \param  bus A bitmask of the bus where the DS1820 is located.
 *  
 *  \param  id  The 64 bit identifier of the DS1820.
 *
 *  \return The 16 bit signed temperature read from the DS1820.
 */
void DS1820_ReadTemperature(unsigned char nr)
{
	unsigned char Feld[9], i, bus, extpwr;
	char remainder, temperature;
	
	eeprom_read_block(Feld,(char *)(nr<<4),8);		// Seriennummer ohne crc mit bus-Maske lesen
	bus = Feld[7] & BUSES;								// Bus-Maske merken und auf geschaltete Ltg. begrenzen
	Feld[7] = OWI_ComputeCRCn(Feld,7);					// CRC ergnzen
	if (Feld[0] != 0x10) DALLAS[nr][1] = 0xff;		// Baustein ist kein Thermometer und Ende
	else {												// Berechnungen fr DS18(s)20 durchfhren
		printf("#%d ",nr); for (i=0; i<8; i++)		// Bausteinindex und
		printf("%02X",Feld[i]); printf(" : ");		// Seriennummer ausgeben
		OWI_DetectPresence(bus);						// Reset.
		OWI_MatchRom(Feld, bus);    					// Match id.
		OWI_SendByte(DS1820_READ_POWER_SUPPLY, bus);	// Send read Power supply command.
		extpwr = OWI_ReadBit(bus);						// Read "0"=parasite power; "1"=external power
		OWI_DetectPresence(bus);						// Reset.
		OWI_MatchRom(Feld, bus);    					// Match id.
		OWI_ReadSCRATCHPAD(Feld, bus);					// Daten auslesen, "Feld" wird nun fr die Daten benutzt !!!
		if(!OWI_ComputeCRCn(Feld,9)&&bus) {			// Wenn crc-Summe der Daten stimmt, Temperatur errechnen
			temperature  = Feld[0] >> 1; 		// temperature low, 0,5 abtrennen
			temperature |= Feld[1] << 7;		// temperature high; Feld[6]=count remain; Feld[7]=count per degree 
			remainder = ((Feld[7]-Feld[6]) * 100) / Feld[7];		// Bruchteil auf 100 skalieren
			if (remainder<25) {temperature--; remainder += 100;}	// Temperatur gfs. korrigieren
			remainder -= 25;					// Nachkommaanteil verschieben
			if (temperature > 127) {			// Falls die Temperatur negativ ist
				temperature = ~temperature;	// Einerkomplement bilden
				if (remainder == 0) temperature++;	// in Zweierkomplement umwandeln
				else remainder = 100 - remainder;		// oder nur den Nachkommaanteil abziehen
				temperature |= 0x80;			// Vorzeichen wieder hinzufgen
			}
			DALLAS[nr][0] = temperature;		// Temperatur in C, ganzzahliger Anteil nach Betrag und Vorzeichen
			DALLAS[nr][1] = remainder;			// Nachkommaanteil in 0,01C
			
			printf("ok T=");				// Testausgabe auf dem Terminal
			if (temperature > 127) {			// Falls die Temperatur negativ ist
				temperature &= 0x7f;			// Vorzeichenbit entfernen
				printf("-");}					// und "-" ausgeben
			printf ("%d,%02d Data=%02X%02X cpdeg=%d remain=%d\n",temperature,remainder,Feld[1],Feld[0],Feld[7],Feld[6]);
			
			if (extpwr!=0) DALLAS[nr][1] |= 0x80;	// "External Power Bit" setzen
		}
		else {
			printf("### Fehler\n");				// crc-Fehler im Datenfeld
			DALLAS[nr][1] = 0x7f;				// Thermometer ist nicht aktiv
		}
	}
}
